# LMH
from __future__ import absolute_import, print_function
"""
# Copyright (C) 2019 Zerodebug
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#

"""

CC_STATUS = 176
NUM_NOTES = 128
NUM_CC_NO = 128
NUM_CHANNELS = 16
JOG_DIAL_CC = 60
RWD_NOTE = 91
FFWD_NOTE = 92
STOP_NOTE = 93
PLAY_NOTE = 94
REC_NOTE = 95
SHIFT_NOTE = 70
FIRE_ONE_CHANNEL = 0

import sys
import Live

if sys.version_info[0] >= 3:
    from .Utils import RemixNet
    import random
    import threading
    import struct
    import time
    import math
    import _thread
    import queue as Queue
    
    from .config import script_version as config_script_version
    from .config import app_main_version as config_app_main_version
    from .config import app_minor_version as config_app_minor_version
    from .config import app_sub_minor_version as config_app_sub_minor_version
    from .Utils import touchAbleUtils
    from .Utils import LiveUtils
    from . import touchAbleCommon
    from . import touchAbleListener
    from . import touchAbleCallbacks
    from threading import Timer
    from collections import defaultdict
    from .Utils.Logger import Logger as Logger
    #from Utils.SessionComponent import TouchableSessionComponent as TouchableSessionComponent 
    #from Utils.SessionRingComponent import SessionRingComponent  # SVH Addition # Zerodebug Addition
    from _Generic.util import DeviceAppointer
    #from _Framework.ControlSurface import ControlSurface
    from inspect import currentframe, getframeinfo



else: 
    import struct
    from .Utils import RemixNet
    from .Utils import OSC
    import time
    import unicodedata
    import thread
    from . import config
    import random
    import threading
    import struct
    import time
    import math
    from .Utils import touchAbleUtils as touchAbleUtils
    from .Utils import LiveUtils as LiveUtils
    from . import  touchAbleCommon
    from . import  touchAbleListener
    from . import  touchAbleCallbacks

    from threading import Timer
    from collections import defaultdict
    from .Utils import Logger as Logger
    from _Generic.util import DeviceAppointer

    from inspect import currentframe, getframeinfo


import logging
from ableton.v2.base import const, inject
from ableton.v2.control_surface import ControlSurface, Layer, MIDI_NOTE_TYPE
from ableton.v2.control_surface.components import DrumGroupComponent, MixerComponent, SessionRecordingComponent, SessionRingComponent
from ableton.v2.control_surface.elements import ButtonMatrixElement
from ableton.v2.control_surface.mode import LayerMode, ModesComponent
from ableton.v2.control_surface.control import ButtonControl
from ableton.v2.base import listens
logger = logging.getLogger(__name__)


# from ableton.v2.base import task
# from ableton.v2.control_surface import Component
# from ableton.v2.control_surface.control.control import InputControl
# PING_PERIOD = 3
# from ableton.v2.control_surface.elements import MultiElement, SysexElement

# from ableton.v2.control_surface.elements import SysexElement
# from functools import partial


# SYSEX_MSG_HEADER = (240, 71, 0)
# SYSEX_END_BYTE = 247
# MPC_X_PRODUCT_ID = 58
# MPC_LIVE_PRODUCT_ID = 59
# FORCE_PRODUCT_ID = 64
# SUPPORTED_PRODUCT_IDS = (MPC_LIVE_PRODUCT_ID, MPC_X_PRODUCT_ID, FORCE_PRODUCT_ID)
# BROADCAST_ID = 127
# PING_MSG_TYPE = 0
# PONG_MSG_TYPE = 1
# TEXT_MSG_TYPE = 16
# COLOR_MSG_TYPE = 17
# PING_MSG = SYSEX_MSG_HEADER + (BROADCAST_ID, PING_MSG_TYPE, SYSEX_END_BYTE)
# INNER_MESSAGE_MAX_LENGTH = 251
# NUM_MSG_IDENTIFYING_BYTES = 3


# class IdentifyingSysexElement(SysexElement):

#     def receive_value(self, _):
#         value = self._msg_sysex_identifier[3]
#         self.notify_value(value)

# class PingPongComponent(Component):
#     pong = InputControl()

#     def __init__(self, on_pong_callback = None, on_ping_timeout = None, *a, **k):
#         super(PingPongComponent, self).__init__(*a, **k)
#         assert on_pong_callback is not None
#         assert on_ping_timeout is not None
#         self._on_pong_callback = on_pong_callback
#         self._on_ping_timeout = on_ping_timeout
#         self._ping = None
#         self._pong_received = False
#         self._keepalive_task = self._tasks.add(task.sequence(task.run(self._send_ping), task.wait(PING_PERIOD), task.run(self._check_pong_received)))
#         self._keepalive_task.kill()
#         self._keepalive_starter_task = self._tasks.add(task.run(self._keepalive_task.restart))
#         self._keepalive_starter_task.kill()

#     @pong.value
#     def pong(self, value, _):
#         self._pong_received = True
#         if not self._keepalive_task.is_running:
#             self._keepalive_starter_task.restart()
#         self._on_pong_callback(value)

#     def set_ping(self, button):
#         self._ping = button
#         self._keepalive_starter_task.restart()

#     def _send_ping(self):
#         if self._ping and self.is_enabled():
#             self._pong_received = False
#             self._ping.send_value(0)

#     def _check_pong_received(self):
#         if not self._pong_received:
#             self._keepalive_task.kill()
#             self._on_ping_timeout()
#         else:
#             self._keepalive_starter_task.restart()

    

class touchAble(ControlSurface):
    __module__ = __name__
    __doc__ = "Main class that establishes the touchAble Component"
    
    # Enable Logging
    _LOG = 0

    # local variables
    updateTime = 0
    long_counter = 0
    script_load = 0
    # globals used live variables
    arrangmentViewSupport = 0
    onetap = 0
    metermode = 1
    received_midi_cmd = 1
    scene = 0
    track = 0
    width = 16
    session = 0
    super_timer = None
    oscServer = None
    if sys.version_info[0] >= 3:
    # Version variables
        script_version = config_script_version
        app_main_version = config_app_main_version
        app_minor_version = config_app_minor_version
        app_sub_minor_version = config_app_sub_minor_version
    else:
        script_version = config.script_version
        app_main_version = config.app_main_version
        app_minor_version = config.app_minor_version
        app_sub_minor_version = config.app_sub_minor_version

    # Script status variables
    did_init_osc_server = 0
    did_init = 0
    listeners_updated = 0

    # Callback and Listener classes
    tACommon = None
    tAListener = None
    tACallbacks = None
    update_counter = 0
    srcPort = 0
    bugfix_version = Live.Application.get_application().get_bugfix_version()
    minor_version = Live.Application.get_application().get_minor_version()
    major_version = Live.Application.get_application().get_major_version()

    control_button = None

    def __init__(self, c_instance):

        

        

        self._touchAble__c_instance = c_instance
        ControlSurface.__init__( self, c_instance )
        #    def __init__(self, *a, **k):
        #super(Blackstar_Live_Logic, self).__init__(*a, **k)
        try:
            self.init_locked_device()
        except:
            self.error_log("init_locked_device failed")

        try:
            self.do_init()
        except:
            self.error_log("do_init failed")  
        
            
        # control_button = ButtonControl()
        self.ta_logger("Control Surface verion " + str(self.script_version),45)


        # with self.component_guard():
        #     self._ping_element = SysexElement(send_message_generator=const(SYSEX_MSG_HEADER + (BROADCAST_ID, PING_MSG_TYPE, SYSEX_END_BYTE)), name=u'Ping_Element')
        #     self._pong_element = MultiElement(name=u'Pong_Multi_Element', *[ IdentifyingSysexElement(sysex_identifier=SYSEX_MSG_HEADER + (identifier, PONG_MSG_TYPE), name=u'Pong_Element_{}'.format(index)) for index, identifier in enumerate(SUPPORTED_PRODUCT_IDS) ])
        #     self._ping_pong = PingPongComponent(on_pong_callback=self.processIncomingUDP, on_ping_timeout=self.processIncomingUDP, name=u'Ping_Pong')
        #     self._ping_pong.layer = Layer(ping=self._ping_element, pong=self._pong_element)

        #thread.start_new_thread(self.do_init())
    def set_rand_vol(self):

        try:
            LiveUtils.trackVolume(2,random.uniform(0, 1))
            song = Live.Application.get_application().get_document()
            if song != None and len(song.track) >= 2:
                atrack = song.tracks[3]
                atrack.mixer_device.volume.value = random.uniform(0, 1)
        except:
            asd = 1

    def set_a_vol(self,aval):

        try:
            LiveUtils.trackVolume(2,aval)
        except:
            pass

    def init_locked_device(self):

        self.locked_device = LiveUtils.getSong().view.selected_track.view.selected_device
        self.lock_to_device(self.locked_device)
        self._device_appointer = None
        self._device_selection_follows_track_selection = True
        
        if self._device_selection_follows_track_selection == True:
            self.log_info_message("devices DO follow track selection..")
        else:
            self.log_info_message("devices DO NOT!!! follow track selection..")

        self.show_message("loading touchAble Pro Control Surface..")

        try:
            self._device_appointer = DeviceAppointer(song=LiveUtils.getSong(), appointed_device_setter=self._set_appointed_device)
        except:
            self.error_log("touchAble: DeviceAppointer init failed")
            pass


    def _log( self, msg, force = False ):
        if self._LOG or force:
            self.ta_logger( msg , 5)

    def get_and_print_tuple(self,in_object):
        if type(in_object) is tuple:
            for sub_object in in_object:
                print(str(in_object))
            self.get_and_print_tuple(in_object)
        else:
            print(str(in_object))



    def do_init(self):

        #self.get_and_print_tuple(dir(Live))

        

        if self.did_init_osc_server == 0:
            self.did_init_osc_server = 1

            # self.oscServer = OSCServer(self, '127.0.0.1', 9111, None, 9008)
            # self.callbackManager = self.oscServer.callbackManager
            
            try:
                self.oscServer = RemixNet.OSCServer(self, '127.0.0.1', 9111, None, 9008)
                self.callbackManager = self.oscServer.callbackManager
            except:
                self.error_log("touchAble: OSCServer init failed")
                self.did_init_osc_server = 0
                self.did_init = 0
                pass
            try:
                self.tACommon = touchAbleCommon.touchAbleCommon(self._touchAble__c_instance, self.oscServer, self)
            except:
                self.did_init = 0
                self.error_log("touchAble: tACommon init failed")
                pass
            try:
                self.tAListener = touchAbleListener.touchAbleListener(self._touchAble__c_instance, self.oscServer, self)
            except:
                self.did_init = 0
                self.error_log("touchAble: tAListener init failed")
                pass
            try:
                self.logger = self._LOG and Logger() or 0
            except:
                self.error_log("touchAble: Logger init failed")
                pass
            try:
                with self.component_guard():
                    pass
                    #self._session_ring = self._create_session()  # SVH Change # Zerodebug Addition
            except:
                self.error_log("touchAble: _create_session failed")

            self.did_init = 1
        else:
            self.did_init = 0
            pass

    #threading.Thread(target=self.dummy_run, args=("a",)).start()




    # def from_dummy_thread(self,func_to_call_from_main_thread):
    #     if self.oscServer != None:
    #         self.oscServer.sendOSC('/server/log', "from_dummy_thread")

    #     callback_queue.put(func_to_call_from_main_thread)

    # def from_main_thread_blocking(self):
    #     callback = callback_queue.get() #blocks until an item is available
    #     callback()

    # def from_main_thread_nonblocking(self):


    #     while True:
    #         if self.oscServer != None:
    #             self.oscServer.sendOSC('/server/log', "from_main_thread_nonblocking")
    #         try:
    #             callback = callback_queue.get(False) #doesn't block
    #         except Queue.Empty: #raised when queue is empty
    #             break
    #         callback()

    # def print_num(self,dummyid, n):
    #     if self.oscServer != None:
    #         self.oscServer.sendOSC('/server/log', "print_num")

    #     if self.oscServer != None:
    #         try:

    #             with self.component_guard():
    #                 self.oscServer.processIncomingUDP()
    #         except:
    #             pass

    # def dummy_run(self,dummyid):

    #     while True:
    #         if self.oscServer != None:
    #             self.oscServer.sendOSC('/server/log', "dummy_run")
    #         self.from_dummy_thread(lambda: self.print_num(dummyid, i))
    #         time.sleep(0.01)  

    def update_all_listenersCB(self,msg):
        if (self.listeners_updated != 1):
            self.listeners_updated = 1
            self.update_all_listeners()
            #thread.start_new_thread(self.update_all_listeners, ())


    def _create_session(self):
        log_text = "_create_session : v " + str(self.script_version)
        session_ring = SessionRingComponent(  # SVH Addition
            num_tracks=8,
            num_scenes=10,
            is_enabled=True,
            is_root=True,
            register_component=self._register_component,
            song=self.song(),
            set_session_highlight=self._set_session_highlight,
        )
        self._log( "_create_session complete", True )
        return session_ring  # SVH Addition # Zerodebug Change


    def set_lsync_offsets(self,x,y,w,h):
        pass
        #self._log( "set_lsync_offsets: " + str( x ) + ", " + str( y ) + ", " + str( w ) + ", " + str( h ) )
        #self.session._set_lsync_offsets( x, y, w, h )
    

    
    def _send_lsync_coords(self):
        pass
        #self._log( "_send_lsync_coords" )
        #self.session._send_lsync_coords( "6" )
        
    #self.log(str(Live.Application.get_application().view.available_main_views()))
        #LiveUtils.getSong().master_track.create_device(str("EQEight"))
    
    def _set_appointed_device(self, device):

        self.unlock_from_device(self.locked_device)

        self.lock_to_device(device)

        self.locked_device = device
                
        pass
    
    def loadAllCB(self, msg):
        """Called when a /live/tracks message is received.

        Messages:
        /live/tracks       no argument or 'query'  Returns the total number of scenes

        """
        if len(msg) == 2 or (len(msg) == 3 and msg[2] == "query"):
            trackTotal = len(LiveUtils.getTracks())
            sceneTotal = len(LiveUtils.getScenes())
            returnsTotal = len(LiveUtils.getSong().return_tracks)
            self.oscServer.sendOSC("/set/size", (trackTotal, sceneTotal, returnsTotal))
            return

    def updateCB(self):
        self.tACommon.positions()
        self.tACommon.mastermeters()
        self.tACommon.meters()
        self.tACommon.songtime_change()
        

    def sendScriptPong(self):

        c = self.bugfix_version
        b = self.minor_version
        a = self.major_version

        xx = int(self.srcPort)

        self.oscServer.sendOSCUDP("/script/pong", (int(self.script_version), int(xx), int(a), int(b), int(c),int(self.listeners_updated),int(self.received_midi_cmd)))
        #self.oscServer.sendOSCUDP("/script/version", self.script_version)
        #self.oscServer.sendOSCUDP("/script/version/str", str(self.script_version))

######################################################################
# Standard Ableton Methods

    def connect_script_instances(self, instanciated_scripts):
        """
        Called by the Application as soon as all scripts are initialized.
        You can connect yourself to other running scripts here, as we do it
        connect the extension modules
        """
        return

    def is_extension(self):
        return False

    def request_rebuild_midi_mapasd(self):

        """
        To be called from any components, as soon as their internal state changed in a 
        way, that we do need to remap the mappings that are processed directly by the 
        Live engine.
        Dont assume that the request will immediately result in a call to
        your build_midi_map function. For performance reasons this is only
        called once per GUI frame.
        """
        return
    
    def update_display_hook(self):
        """
        This function is run every 100ms, so we use it to initiate our Song.current_song_time
        listener to allow us to process incoming OSC commands as quickly as possible under
        the current listener scheme.
        """
        ######################################################
        # START OSC LISTENER SETUPx



        if self.tACallbacks == None and self.did_init == 1:



            # By default we have set tACallbacks to 0 so that we can assign it after
            # initialization. We try to get the current song and if we can we'll
            # connect our tACallbacks callbacks to the listener allowing us to 
            # respond to incoming OSC every 60ms.
            #
            # Since this method is called every 100ms regardless of the song time
            # changing, we use both methods for processing incoming UDP requests
            # so that from a resting state you can initiate play/clip triggering.
            

            try:
                # LMH

                self.tACallbacks = touchAbleCallbacks.touchAbleCallbacks(self._touchAble__c_instance, self.oscServer, self)

                # Commented for stability
                #doc.add_current_song_time_listener(self.oscServer.processIncomingUDP)
                #self.oscServer.sendOSC("/server/refresh", (1))

                
            except:
                self.error_log("could not create touchAbleCallbacks")
                return
    
            
            # If our OSC server is listening, try processing incoming requests.
            # Any 'play' initiation will trigger the current_song_time listener
            # and bump updates from 100ms to 60ms.

        

                 

        #self.updateTime = self.updateTime +4
        #if self.updateTime >= 100:
      
        #self.set_a_vol((1 + math.sin(round(time.time() * 1000.0)/1000.))/2.0)
        if self.oscServer != None:
            #threading.Timer(1, self.on_a_timer).start()
            self.sendScriptPong()
            self.updateCB()
            #self.oscServer.sendOSC('/midi/send_midi',1)

        #self.long_counter = self.long_counter + 1
        # if self.tACallbacks != None and self.script_load == 1 and self.did_init == 1:
        #     self.script_load = 0
        #     self.tACallbacks.scriptLoadCB(msg=1)
        #     self.oscServer.sendOSC('/server/log', "scriptLoadCB")

        if self.oscServer != None:
            try:
                #self.oscServer.startNewThread()
                #if self.oscServer.tcpServer.new_thread != None:
                #    self.oscServer.tcpServer.new_thread.wait()

                with self.component_guard():
                    self.oscServer.processIncomingUDP()


                #Timer(0.02, self.update_display()).start()
                #t.start()  # after 30 seconds, "hello, world" will be printed
                #self.show_message("long counter: " + str(self.long_counter) + " thread runnig " + str(self.oscServer.tcpServer.second_thread_running))
                #if self.oscServer.tcpServer.new_thread != None:
                #    self.oscServer.tcpServer.new_thread.notify()
            except:
                pass
        
        


        # END OSC LISTENER SETUP
        ######################################################

    def send_midi(self, midi_event_bytes):
        #self.oscServer.sendOSC('/server/log', "send_midi")
        pass
        """
        Use this function to send MIDI events through Live to the _real_ MIDI devices 
        that this script is assigned to.
        """
        #self._touchAble__c_instance.send_midi(midi_event_bytes)
        
    def receive_midi_cc(self,midi_bytes):
        #self.oscServer.sendOSC('/server/log', "receive_midicc")
        pass

    def receive_midi(self,midi_bytes):
        
        #a = 1
        #a = midi_bytes[0] & 240
        #self.oscServer.sendOSC('/server/log', "receive_midi")
        if midi_bytes[0] == 176 and self.did_init == 1:
            self.received_midi_cmd = 1
            if midi_bytes[1] == 3 and self.oscServer:
                self.updateCB()
                #self.oscServer.sendOSC('/midi/received', (midi_bytes))
            elif self.oscServer:
                try:
                    with self.component_guard():
                        self.oscServer.processIncomingUDP()
                    #threading.Timer(0.00, self.on_a_timer).start()
                except:
                    pass
        else:
            pass


    def processIncomingUDP(self):

        try:
            if self.oscServer.processIncomingUDPRunning() == False:
                self.oscServer.processIncomingUDP()
        except:
            return
                  
        
        return

    def process_midi_bytes(self, midi_bytes, midi_processor):
        # self.error_logger("process_midi_bytes..")
        #super(touchAble, self).process_midi_bytes(midi_bytes, midi_processor)
        #self.notify_received_midi(*midi_bytes)
        #self.show_message("in udp " + str(threading.current_thread()))
        #self.oscServer.sendOSC('/server/log', "process_midi_bytes")
        """
        The switches send sysex which we need to ignore to avoid
        an error being logged from Live
        """
        #return super(touchAble, self).process_midi_bytes(midi_bytes, midi_processor)
        # LiveUtils.getSong().master_track.mixer_device.volume.value = 1
        # self.refresh_state()
        # LiveUtils.getSong().undo()
        #self.error_logger("process_midi_bytes from device..")
        try:
            self.processIncomingUDP()
        except:
            return
                  
        # return super(touchAble, self).process_midi_bytes(midi_bytes, midi_processor)
        
        return
            
    def can_lock_to_devices(self):
        return True
    
    def lock_to_device(self, device):
        ControlSurface.lock_to_device(self, device)
        #self.ta_logger("locking to device..",5)
        pass

    def unlock_from_device(self, device):
        ControlSurface.unlock_from_device(self, device)
        #self.ta_logger("unlocking from device..",5)
        pass

    def suggest_input_port(self):
        return 'touchAble Script Input'

    def suggest_output_port(self):
        return 'touchAble Script Output'

    def suggest_map_mode(self, cc_no, channel):
        return Live.MidiMap.MapMode.absolute
    

    
######################################################################
# Useful Methods


    def getOSCServer(self):
        return self.oscServer
    
    def application(self):
        """returns a reference to the application that we are running in"""
        return Live.Application.get_application()


    def handle(self):
        """returns a handle to the c_interface that is needed when forwarding MIDI events via the MIDI map"""
        return self._touchAble__c_instance.handle()

    def log(self, msg):
        if self._LOG == 1:
            self.logger.log(msg) 
            
        
######################################################################
# Used Ableton Methods

    def disconnect(self):

        try:
            #self.tAListener.remove_all_listeners()            
            self.oscServer.sendOSC('/remix/oscserver/shutdown', 1)
            self.oscServer.shutdown()
        except:
            self.oscServer.sendOSC('/remix/oscserver/shutdown', 1)
            self.oscServer.shutdown()


    #def request_rebuild_midi_map(self):
    #    self._touchAble__c_instance.request_rebuild_midi_map()

    def build_midi_map(self, midi_map_handle):
        
        # self.control_button.connect_to(LiveUtils.getSong().master_track.mixer_device.volume)
        Live.MidiMap.forward_midi_cc(self.handle(), midi_map_handle, 0, 2, False)
        # feedback_rule = Live.MidiMap.CCFeedbackRule()
        # Live.MidiMap.map_midi_cc_with_feedback_map(midi_map_handle, LiveUtils.getSong().master_track.mixer_device.volume, 0, 2, Live.MidiMap.MapMode.absolute, feedback_rule, False)
        # Live.MidiMap.send_feedback_for_parameter(midi_map_handle, LiveUtils.getSong().master_track.mixer_device.volume)
        return
        #feedback_rule = Live.MidiMap.CCFeedbackRule()
        #feedback_rule.cc_no = 2
        #Live.MidiMap.map_midi_cc_with_feedback_map(midi_map_handle, LiveUtils.getSong().master_track.mixer_device.volume, 0, 2, Live.MidiMap.MapMode.absolute, feedback_rule, False)
        #Live.MidiMap.map_midi_cc(midi_map_handle, LiveUtils.getSong().master_track.mixer_device.volume, 0, 2, Live.MidiMap.MapMode.absolute, False)
        #Live.MidiMap.send_feedback_for_parameter(midi_map_handle, LiveUtils.getSong().master_track.mixer_device.volume)
        #Live.MidiMap.map_midi_cc(self.handle(), midi_map_handle, 0, 2, 0)
        #Live.MidiMap.send_feedback_for_parameter(midi_map_handle, self.__fader_parameter)
        #Live.MidiMap.map_midi_pitchbend_with_feedback_map(midi_map_handle, parameter, control.message_channel(), feedback_rule, not control.needs_takeover())
        #Live.MidiMap.forward_midi_pitchbend(self._touchAble__c_instance.handle(), midi_map_handle, 0)

    def update_all_listeners(self):
        self.tAListener.update_all_listeners()

        
    def log_message(self,msg):
        logger.warning(str(msg))


    def log_info_message(self,msg):
        logger.info(str(msg))

    def ta_logger(self, msg, number=1):
        current_log_level = 0
        if number > current_log_level:
            frameinfo = getframeinfo(currentframe())
            self.log_info_message("#### touchAble " +str(self.script_version)+ " TA LOG: " + " line:"+ str(frameinfo.lineno) + " msg: " + msg + " ###")
            send_to_app = 0
            if send_to_app == 1:
                self.oscServer.sendOSCUDP("/LiveScriptLog", msg)

    def error_logger(self, msg):
        frameinfo = getframeinfo(currentframe())

        self.log_message("*** touchAble " +str(self.script_version)+ " Script Error: " + " line:"+ str(frameinfo.lineno) + " msg: " + msg + " ***")
        send_to_app = 0
        if send_to_app == 1:
            self.oscServer.sendOSCUDP("/LiveScriptLog", msg)

    def error_log(self, msg):
        frameinfo = getframeinfo(currentframe())

        self.log_message("*** touchAble " +str(self.script_version)+ " Script Error: " + " line:"+ str(frameinfo.lineno) + " msg: " + msg + " ***")
        send_to_app = 0
        if send_to_app == 1:
            self.oscServer.sendOSCUDP("/LiveScriptLog", msg)



